local tec = TeamEquipCompare;
tec_lineAdded = false;

function TEC_OnTooltipSetItem(tooltip, ...)
	if (TEC_IsTooltipActive()) then
		local ttData = TEC_GetTooltipItemData(tooltip);

		-- ignore non-gear items
		if (ttData.slot and ttData.slot.ids) then
			TEC_AddHeader(tooltip);

			-- loop thru sorted list of team members
			local sortedNames = tec_keys(tec.party, true);
			for _, unitName in pairs(sortedNames) do
				local unit = tec.party[unitName];
				if (TEC_DataIsComplete(unit)) then
					local isUsable = TEC_AddPlayerLine(tooltip, unit, ttData);

					if (isUsable) then
						TEC_AddStatLines(tooltip, ttData, unit);
						tooltip:AddLine(" ");
					end
				end
			end
		end
		
		tec_lineAdded = true;
	end
end

function TEC_OnTooltipCleared(tooltip, ...)
	tec_lineAdded = false;
end

function TEC_GetTooltipItemData(tooltip)
	local t = {};

	-- get info for tooltip item
	t.name, t.link = tooltip:GetItem();
	_, _, _, _, t.reqLevel, t.class, t.subclass, _, t.equipSlot, _, _ = GetItemInfo(t.link);
	t.slot = TEC_Slot_Lookup[t.equipSlot];
	
	return t;
end

function TEC_AddHeader(tooltip)
	-- add heading line with spacing
	tooltip:AddLine(" ");
	if (not IsInGroup()) then
		-- simple message if player is not in a group
		tooltip:AddDoubleLine("Team Equip Compare:", "|cffff0000[Not in Group]|r");
		tooltip:AddLine(" ");
		return;
	else
		tooltip:AddLine("Team Equip Compare:");
		tooltip:AddLine(" ");
	end
end

function TEC_AddPlayerLine(tooltip, unit, ttData)
	local class = TEC_Class_Lookup[unit.class];
	local classColor = TEC_ClassColor_Lookup[unit.class];
	local isUsable, msg = TEC_IsItemUsable(unit, class, ttData.reqLevel, ttData.class, ttData.subclass);
	local playerTag = format("|c%s%s (%s: %d, iLevel: %d):|r", classColor, unit.name, unit.spec, unit.level, unit.ilevel.equipped);

	if (isUsable) then
		-- add party member heading
		tooltip:AddLine(playerTag);
	else
		-- add player name with note that tooltip item is unusable
		tooltip:AddDoubleLine(playerTag, format("|cffff0000[%s]|r", msg));
		tooltip:AddLine(" ");
	end
	
	return isUsable;
end

function TEC_AddStatLines(tooltip, ttData, unit)
	-- compare item with each potential slot (weapons, trinkets and fingers have two slots)
	for ii = 1, #ttData.slot.ids do
		-- grab item from syncronized party member data
		local link = unit.gear[ttData.slot.ids[ii]];
		if (link) then
			-- get party member item data
			local name, _, quality, ilevel, _, _, _, _, _, _, _ = GetItemInfo(link);
			if (name) then
				local itemTag = format("    |c%s[%d] %s|r", TEC_QualityColor_Lookup[quality], ilevel, name)
				local statLine1, statLine2 = TEC_GetStatLines(ttData.link, link);

				tooltip:AddDoubleLine(itemTag, statLine1);
				if (statLine2) and (#statLine2 > 0) then
					tooltip:AddDoubleLine(" ", statLine2);
				end
			end
		end
	end
end

function TEC_GetStatLines(itemLink, equippedLink)
	local line1 = "";
	local line2 = "";

	local statDelta = GetItemStatDelta(itemLink, equippedLink);
	local count = 0;
	for stat, value in pairs(statDelta) do
		-- get friendly name string from lookup table
		statName = TEC_Stat_Lookup[stat];
		if (not statName) then statName = tostring(stat) end

		-- color comparison text depending on +/-
		if (value > 0) then
			if (#line1 > 0) then line1 = line1 .. ", " end
			line1 = line1..format("|cff00ff00+%d|r %s", value, statName);
			count = count + 1;
		elseif (value < 0) then
			if (#line2 > 0) then line2 = line2 .. ", " end
			line2 = line2..format("|cffff0000%d|r %s", value, statName);
			count = count + 1;
		end
	end

	-- make adjustments based on how much was added
	if (#line1 == 0) and (#line2 == 0) then
		line1 = "[Identical]";
	elseif (#line1 == 0) then
		-- if there are only negative values, shift it up a line
		line1 = line2;
		line2 = "";
	elseif (count < 5) then
		line1 = line1..", "..line2;
		line2 = "";
	end

	return line1, line2;
end

function TEC_IsTooltipActive()
	-- check if the line has been added already
	if (tec_lineAdded) then return false end

	-- check if we require ctrl key
	if (not tec.db.global.requireCtrlKey) then return true end
	
	-- otherwise, check if ctrl key is pressed
	if (IsControlKeyDown()) then return true end

	-- if we got here, it's not active
	return false;
end

function TEC_DataIsComplete(unit)
	-- make sure we have all the data we need
	-- (there is sometimes a little lag/hiccup resulting in errors)
	if (not unit)
			or (not unit.name)
			or (not unit.class)
			or (not unit.level)
			or (not unit.spec)
			or not (unit.gear)
			or (not unit.ilevel)
			or (not unit.ilevel.equipped) then
		return false;
	end
	
	return true;
end

function TEC_IsItemUsable(unit, class, reqLevel, itemClass, itemSubclass)
	local msg = "";

	if (not tec.db.global.hideUnusable) then
		msg = "";
	elseif (tec.db.global.levelCheck) and (reqLevel > unit.level) then
		msg = "Required level "..reqLevel;
	elseif (not class) then
		msg = "Unable to equip";
	elseif (not class[itemClass]) then
		msg = "Unable to equip";
	elseif (not class[itemClass][itemSubclass]) then
		msg = "Unable to equip "..itemSubclass;
	elseif (tec.db.global.levelCheck) and (class[itemClass][itemSubclass] > unit.level) then
		msg = "Required level "..class[itemClass][itemSubclass];
	end

	return (#msg == 0), msg;
end

-- hook into the game tooltip
GameTooltip:HookScript("OnTooltipSetItem", TEC_OnTooltipSetItem);
GameTooltip:HookScript("OnTooltipCleared", TEC_OnTooltipCleared);